program checkoutput
!use realsame_mod
use dal
use errorhandling

implicit none

type(DataSetT)  :: inevents
type(TableT)    :: bpixtable
type(ColumnT)   :: xCol, yCol, badflagCol
integer(int16), dimension(:), pointer ::p_x, p_y, p_badflag
integer i,nrows
logical ok

ok = .true. 
!-------------------------------------------------------------------------
!	PN
write (*,*) 'testing PN output'
inevents = dataSet("pn.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) <= 0 .or. p_y(i) <= 0 .or. &
      p_x(i) >= 65 .or. p_y(i) >= 201) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","PN Coordinates out of range")
ok = .true.
if (p_x(0) .ne. 1 .or. p_y(0) .ne. 2) ok = .false.
if (.not.ok) call error("wrongValues","PN Coordinate values wrong")
ok = .true.
call release(inevents)
!_____________________________________________________________________
!	MOS 
write(*,*) 'testing MOS output'
inevents = dataSet("mos.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","MOS Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","MOS Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) < 1.0 .or. p_y(i) < 1.0 .or. &
      p_x(i) > 600 .or. p_y(i) > 600) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","MOS Coordinates out of range")
ok = .true.
if (p_x(0) .ne. 171 .or. p_y(0) .ne. 27) ok = .false.
if (.not.ok) call error("wrongValues","MOS Coordinate values wrong")
ok = .true.
call release(inevents)
!!!_____________________________________________________________________
!	PN (mode3)
write (*,*) 'testing output (just mode3)' 
inevents = dataSet("pn3.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN3 Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN3 Column absent or incorrectly named")
ok = .true.
badflagCol = column(bpixtable, "BADFLAG",read)
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_badflag => int16Data(badflagCol)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_badflag(i) .ne. 3) ok = .false.
enddo
if (.not.ok) call error("wrongBadflag","PN3 Non-mode3 pixels exist")
ok = .true.
if (p_x(0) .ne. 30 .or. p_y(0) .ne. 117) ok = .false.
if (.not.ok) call error("wrongValues","PN3 Coordinate values (badpixfind file) wrong")
ok = .true. 
call release(inevents)
!_____________________________________________________________________
!	PN (emptyext)
write (*,*) 'testing output (empty extension)' 
inevents = dataSet("pn_BU.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN3 Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN3 Column absent or incorrectly named")
ok = .true.
nrows = numberofRows(bpixtable)
if (nrows .ne. 0) ok = .false.
if (.not.ok) call error("wrongExt","extension not empty")
ok = .true.
call release(inevents)
!-------------------------------------------------------------------------
!	PN (window)
write (*,*) 'testing output - window mode'
inevents = dataSet("pn_SW.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) < 1.0 .or. p_y(i) < 1.0 .or. &
      p_x(i) > 64 .or. p_y(i) > 200) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","PN (W) Coordinates out of range")
ok = .true.
if (p_x(1) .ne. 32 .or. p_y(1) .ne. 81) ok = .false.
if (.not.ok) call error("wrongValues","PN (W) Coordinate values wrong")
ok = .true.
call release(inevents)
!_____________________________________________________________________
!	PN (ccf/bpxf purge)
write (*,*) 'testing output - ccf/bpxf purge mode'
inevents = dataSet("pn_Pu_W.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) < 1.0 .or. p_y(i) < 1.0 .or. &
      p_x(i) > 64 .or. p_y(i) > 200) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","PN (CCF/BPXF) Coordinates out of range")
ok = .true.
if (p_x(0) .ne. 15 .or. p_y(0) .ne. 67) ok = .false.
if (.not.ok) call error("wrongValues","PN (CCF/BPXF) Coordinate values wrong")
ok = .true.
call release(inevents)
!_____________________________________________________________________
!	MOS (OAL M1)
write (*,*) 'testing output - MOS1 +OAL '
inevents = dataSet("mos1_3.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) < 1.0 .or. p_y(i) < 1.0 .or. &
      p_x(i) > 600 .or. p_y(i) > 600) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","MOS1 (OAL) Coordinates out of range")
ok = .true.
if (p_x(0) .ne. 158 .or. p_y(0) .ne. 51) ok = .false.
if (.not.ok) call error("wrongValues","MOS1 (OAL) Coordinate values wrong")
ok = .true.
call release(inevents)
!_____________________________________________________________________
!	MOS (OAL M2)
write (*,*) 'testing output - MOS2 +OAL '
inevents = dataSet("mos2_1.in", read)
if (.not.hasBlock(inevents,"BADPIX")) ok = .false. 
if (.not.ok) call error("noExt","PN Extension absent or incorrectly named")
ok = .true.
bpixtable = table(inevents,"BADPIX")
if (.not.hasColumn(bpixtable,"RAWX") .or. &
    .not.hasColumn(bpixtable,"RAWY") .or. &
    .not.hasColumn(bpixtable,"TYPE") .or. &
    .not.hasColumn(bpixtable,"YEXTENT") .or. &
    .not.hasColumn(bpixtable,"BADFLAG")) ok = .false.
if (.not.ok) call error("noColumn","PN Column absent or incorrectly named")
ok = .true.
xCol = column(bpixtable, "RAWX",read)
yCol = column(bpixtable, "RAWY",read)
nrows = numberofRows(bpixtable)
p_x => int16Data(xCol)
p_y => int16Data(yCol)
do i = 0,  nrows-1
  if (p_x(i) < 1.0 .or. p_y(i) < 1.0 .or. &
      p_x(i) > 600 .or. p_y(i) > 600) ok = .false.
enddo
if (.not.ok) call error("wrongCoords","MOS2 (OAL) Coordinates out of range")
ok = .true.
if (p_x(0) .ne. 3 .or. p_y(0) .ne. 393) ok = .false.
if (.not.ok) call error("wrongValues","MOS2 (OAL) Coordinate values wrong")
ok = .true.
call release(inevents)
!_____________________________________________________________________

end

